@@ -1867,6 +1867,18 @@ class VariableClUpdateUseMigrationHost(Variable):
     type = "bool"
     value = "off"
 
+class VariableClUpdateMigrationHost(Variable):
+    """
+    Используемый хост миграции
+    """
+    value = ""
+
+class VariableClBinhostStash(Variable):
+    """
+    Бекап хоста обновления
+    """
+    value = ""
+
 class VariableClCheckUpdateLevel(Variable):
     """
     Проверять уровень обновления 
